@@ -1,6 +1,7 @@
 /*
  * osmo-pcap-client code
  *
+ * (C) 2025 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * (C) 2011-2016 by Holger Hans Peter Freyther <holger@moiji-mobile.com>
  * (C) 2011 by On-Waves
  * All Rights Reserved
@@ -40,7 +41,7 @@ static const struct value_string osmopcap_protocol_names[] = {
 static struct osmo_pcap_client_conn *get_conn(struct vty *vty)
 {
 	if (vty->node == CLIENT_NODE)
-		return &pcap_client->conn;
+		return pcap_client->conn;
 	return vty->index;
 }
 
@@ -115,6 +116,8 @@ static int config_write_server(struct vty *vty)
 	struct osmo_pcap_client_conn *conn;
 
 	llist_for_each_entry(conn, &pcap_client->conns, entry) {
+		if (conn == pcap_client->conn)
+			continue;
 		vty_out(vty, " pcap-store-connection %s%s", conn->name, VTY_NEWLINE);
 		write_client_conn_data(vty, conn, " ");
 		vty_out(vty, "  connect%s", VTY_NEWLINE);
@@ -124,11 +127,16 @@ static int config_write_server(struct vty *vty)
 
 static int config_write_client(struct vty *vty)
 {
+	struct osmo_pcap_handle *ph;
+
 	vty_out(vty, "client%s", VTY_NEWLINE);
 
-	if (pcap_client->device)
+	if (pcap_client->pcap_fmt != OSMO_PCAP_FMT_PCAP)
+		vty_out(vty, " pcap file-format pcapng%s", VTY_NEWLINE);
+	llist_for_each_entry(ph, &pcap_client->handles, entry) {
 		vty_out(vty, " pcap device %s%s",
-			pcap_client->device, VTY_NEWLINE);
+			ph->devname, VTY_NEWLINE);
+	}
 	if (pcap_client->snaplen != DEFAULT_SNAPLEN)
 		vty_out(vty, " pcap snaplen %d%s",
 			pcap_client->snaplen, VTY_NEWLINE);
@@ -141,7 +149,37 @@ static int config_write_client(struct vty *vty)
 		vty_out(vty, " pcap add-filter gprs%s", VTY_NEWLINE);
 
 
-	write_client_conn_data(vty, &pcap_client->conn, "");
+	write_client_conn_data(vty, pcap_client->conn, "");
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_client_pcap_file_format,
+      cfg_client_pcap_file_format_cmd,
+      "pcap file-format (pcap|pcapng)",
+      PCAP_STRING "The pcap file format to use\n"
+      "Libpcap Capture File Format (.pcap)\n"
+      "PCAP Next Generation Capture File Format (.pcapng)\n")
+{
+	if (strcmp(argv[0], "pcap") == 0)
+		pcap_client->pcap_fmt = OSMO_PCAP_FMT_PCAP;
+	else if (strcmp(argv[0], "pcapng") == 0)
+		pcap_client->pcap_fmt = OSMO_PCAP_FMT_PCAPNG;
+	else
+		return CMD_WARNING;
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_client_no_device,
+      cfg_client_no_device_cmd,
+      "no pcap device NAME",
+      NO_STR PCAP_STRING "the device to filter\n" "device name\n")
+{
+	struct osmo_pcap_handle *ph = osmo_client_find_handle(pcap_client, argv[0]);
+	if (!ph) {
+		vty_out(vty, "%% Device %s not found!%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+	osmo_pcap_handle_free(ph);
 	return CMD_SUCCESS;
 }
 
@@ -150,7 +188,9 @@ DEFUN(cfg_client_device,
       "pcap device NAME",
       PCAP_STRING "the device to filter\n" "device name\n")
 {
-	osmo_client_capture(pcap_client, argv[0]);
+	struct osmo_pcap_handle *ph = osmo_client_find_handle(pcap_client, argv[0]);
+	if (!ph)
+		osmo_pcap_handle_alloc(pcap_client, argv[0]);
 	return CMD_SUCCESS;
 }
 
@@ -159,10 +199,6 @@ DEFUN(cfg_client_snaplen,
 	      "pcap snaplen <1-262144>", /* MAXIMUM_SNAPLEN */
       PCAP_STRING "snapshot length\n" "Bytes\n")
 {
-	if (pcap_client->handle) {
-		vty_out(vty, "'pcap snaplen' must be set before 'pcap device' to take effect!%s", VTY_NEWLINE);
-		return CMD_WARNING;
-	}
 	pcap_client->snaplen = atoi(argv[0]);
 	return CMD_SUCCESS;
 }
@@ -228,7 +264,7 @@ DEFUN(cfg_enable_tls,
 
 	if (!conn->tls_on) {
 		if (conn->wqueue.bfd.fd >= 0)
-			osmo_client_reconnect(conn);
+			osmo_client_conn_reconnect(conn);
 	}
 
 	conn->tls_on = true;
@@ -243,7 +279,7 @@ DEFUN(cfg_disable_tls,
 	struct osmo_pcap_client_conn *conn = get_conn(vty);
 
 	if (conn->tls_on)
-		osmo_client_reconnect(conn);
+		osmo_client_conn_reconnect(conn);
 
 	conn->tls_on = false;
 	return CMD_SUCCESS;
@@ -445,7 +481,7 @@ DEFUN(cfg_pcap_store,
 {
 	struct osmo_pcap_client_conn *conn;
 	conn = osmo_client_find_or_create_conn(pcap_client, argv[0]);
-	if (!conn) {
+	if (!conn || conn == pcap_client->conn) {
 		vty_out(vty, "%%Failed to find/create conection %s%s",
 			argv[0], VTY_NEWLINE);
 		return CMD_WARNING;
@@ -463,13 +499,13 @@ DEFUN(cfg_no_pcap_store,
 {
 	struct osmo_pcap_client_conn *conn;
 	conn = osmo_client_find_conn(pcap_client, argv[0]);
-	if (!conn) {
+	if (!conn || conn == pcap_client->conn) {
 		vty_out(vty, "%%Failed to find connection %s%ss",
 			argv[0], VTY_NEWLINE);
 		return CMD_WARNING;
 	}
 
-	osmo_client_free(conn);
+	osmo_client_conn_free(conn);
 	return CMD_SUCCESS;
 }
 
@@ -480,7 +516,7 @@ DEFUN(cfg_client_connect,
 {
 	struct osmo_pcap_client_conn *conn = get_conn(vty);
 
-	osmo_client_connect(conn);
+	osmo_client_conn_connect(conn);
 	return CMD_SUCCESS;
 }
 
@@ -491,7 +527,7 @@ DEFUN(cfg_client_disconnect,
 {
 	struct osmo_pcap_client_conn *conn = get_conn(vty);
 
-	osmo_client_disconnect(conn);
+	osmo_client_conn_disconnect(conn);
 	return CMD_SUCCESS;
 }
 
@@ -542,6 +578,8 @@ int vty_client_init(void)
 
 	install_node(&server_node, config_write_server);
 
+	install_element(CLIENT_NODE, &cfg_client_pcap_file_format_cmd);
+	install_element(CLIENT_NODE, &cfg_client_no_device_cmd);
 	install_element(CLIENT_NODE, &cfg_client_device_cmd);
 	install_element(CLIENT_NODE, &cfg_client_snaplen_cmd);
 	install_element(CLIENT_NODE, &cfg_client_filter_cmd);
